/*
* 
* Copyright (c) 2010 Stuart Langridge
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
* Website: http://code.google.com/p/twidenash/
*/

/*
* With modification by Michael Hall for use in loco.ubuntu.com
*/

twidenash = {
    init: function() {
        if (!twidenash.hashtag) return;
        // request hashtag search values for both twitter and identica
        twidenash.callbacks = 0;
        twidenash.items = [];
        twidenash.add_script("http://locoteams.status.net/api/search.json?" +
           "callback=twidenash.cb&rpp=10&q=%23" + twidenash.hashtag);
        twidenash.add_script("http://identi.ca/api/search.json?" +
           "callback=twidenash.cb&rpp=10&q=%23" + twidenash.hashtag);
        twidenash.add_script("http://search.twitter.com/search.json?" +
           "callback=twidenash.cb&rpp=10&q=%23" + twidenash.hashtag);
        twidenash.printed = 0;
    },
    add_script: function(url) {
        var scr = document.createElement("script");
        scr.src = url;
        document.getElementsByTagName("head")[0].appendChild(scr);
    },
    cb: function(data) {
        for (var i=0; i<data.results.length; i++) {
            var dupe = false;
            for (var j=0; j<twidenash.items.length; j++) {
                // Strip characters that may be specific to networks
                var comp1 = data.results[i].text.replace(/[♺\!\#\@]/g, '');
                var comp2 = twidenash.items[j].text.replace(/[♺\!\#\@]/g, '');
                var compdate = Date.parse(data.results[i].created_at);
                var datediff = Math.abs(Date.parse(data.results[i].created_at) - twidenash.items[j].dt)
                if (comp1 == comp2 && datediff < 120000) {
                    dupe = true;
                    break;
                }
            }
            if (dupe) continue;
            twidenash.items.push({ text: data.results[i].text, 
                img: data.results[i].profile_image_url, 
                dt: Date.parse(data.results[i].created_at),
                user: data.results[i].from_user })
        }
        twidenash.callbacks += 1;
        if (twidenash.callbacks == 2) {
            twidenash.items.sort(function(a,b) { return b.dt - a.dt });
            var ul = document.createElement("ul");
            ul.className = "twidenash";
            for (i=0; i<twidenash.items.length && twidenash.printed < 5; i++) {
                var li = document.createElement("li");
                var img = document.createElement("img");
                img.src = twidenash.items[i].img;
                img.width = 48;
                var span = document.createElement("span");
                span.className = 'comment';
                var nick = document.createElement("span");
                nick.className = 'nick';
                nick.appendChild(document.createTextNode(twidenash.items[i].user + 
                    ": "));
                var exp = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/gi;
                span.innerHTML = twidenash.items[i].text.replace(exp,"<a href='$1' target='_blank'>$1</a>");
                li.appendChild(img);
                li.appendChild(nick);
                li.appendChild(span);
                ul.appendChild(li);
                twidenash.printed++;
            }
            twidenash.scriptelement.parentNode.insertBefore(ul, twidenash.scriptelement);
        }
    },
    cache_scriptname: function() {
        // called before page load so we can find our own name
        var scr = document.getElementsByTagName("script");
        var url = scr[scr.length-1].getAttribute("src");
        if (!url) {
            var twidenash_script = document.getElementById("twidenash_script");
            if (twidenash_script) url = twidenash_script.getAttribute("src");
        }
        if (url && url.lastIndexOf('?') != -1) {
            twidenash.hashtag = url.substr(url.lastIndexOf('?')+1);
            if (twidenash_script) {
                twidenash.scriptelement = twidenash_script;
            } else {
                twidenash.scriptelement = scr[scr.length-1];
            }
        }
    }
};
(function(i) {var u =navigator.userAgent;var e=/*@cc_on!@*/false; var st = 
setTimeout;if(/webkit/i.test(u)){st(function(){var dr=document.readyState;
if(dr=="loaded"||dr=="complete"){i()}else{st(arguments.callee,10);}},10);}
else if((/mozilla/i.test(u)&&!/(compati)/.test(u)) || (/opera/i.test(u))){
document.addEventListener("DOMContentLoaded",i,false); } else if(e){     (
function(){var t=document.createElement('doc:rdy');try{t.doScroll('left');
i();t=null;}catch(e){st(arguments.callee,0);}})();}else{window.onload=i;}})(twidenash.init);
twidenash.cache_scriptname();

