# Ubuntu Developer Summit web application
# Copyright (C) 2008, 2009, 2010 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import datetime
import unittest
from django.core.management.base import CommandError

from summit.schedule.management.commands import reschedule
from summit.schedule.models import *

from model_mommy import mommy as factory
from summit.schedule.fields import NameField

# Monkey-patch our NameField into the types of fields that the factory
# understands.  This is simpler than trying to subclass the Mommy
# class directly.
factory.default_mapping[NameField] = str


class RescheduleCommandTestCase(unittest.TestCase):
    """Tests for the 'reschedule' management command."""

    def test_passing_nonexistant_summit_raises_error(self):
        cmd = reschedule.Command()
        self.assertRaises(CommandError, cmd.handle, summit='uds-xxx')


class MeetingModelTestCase(unittest.TestCase):
    """Tests for the Meeting model."""

    def test_meetings_can_not_be_scheduled_in_closed_slots(self):
        now = datetime.datetime.utcnow()
        one_hour = datetime.timedelta(0, 3600)
        slot = factory.make_one(
            Slot,
            start_utc=now,
            end_utc=now+one_hour,
            type='closed')

        agenda = factory.make_one(Agenda, slot=slot)
        meeting = factory.make_one(Meeting)

        # XXX check_schedule should only be checking the schedule, not
        # checking and modifying the schedule.
        # XXX check_schedule's parameters should be an just an agenda object,
        # not the agenda object's attributes.
        self.assertRaises(
            Meeting.SchedulingError,
            meeting.check_schedule, agenda.slot, agenda.room)
